﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 리스트 박스 컨트롤
    /// </summary>
    public partial class ListBoxControl : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 값 컬럼명
        /// </summary>
        private static string _valueColumnName = "VALUE";

        /// <summary>
        /// 텍스트 컬럼명
        /// </summary>
        private static string _textColumnName = "TEXT";

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 값 멤버
        /// </summary>
        private string valueMember = null;

        /// <summary>
        /// 디스플레이 멤버
        /// </summary>
        private string displayMember = null;

        /// <summary>
        /// 체크 딕셔너리
        /// </summary>
        private Dictionary<int, object> checkDictionary = null;

        /// <summary>
        /// 마우스 드래그 여부
        /// </summary>
        private bool isMouseDragging = false;

        /// <summary>
        /// 드래그 시작 인덱스
        /// </summary>
        private int dragStartIndex;

        /// <summary>
        /// 드래그 종료 인덱스
        /// </summary>
        private int dragEndIndex;

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        private bool readOnly = false;

        /// <summary>
        /// 단일 선택 여부
        /// </summary>
        private bool singleSelect = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 멤버 - ValueMember

        /// <summary>
        /// 값 멤버
        /// </summary>
        public string ValueMember
        {
            get
            {
                return this.valueMember;
            }
            set
            {
                if(CheckSameColumnName(value))
                {
                    return;
                }

                this.valueMember = value;

                RebuildColumns();
            }
        }

        #endregion
        #region 디스플레이 멤버 - DisplayMember

        /// <summary>
        /// 디스플레이 멤버
        /// </summary>
        public string DisplayMember
        {
            get
            {
                return this.displayMember;
            }
            set
            {
                if(CheckSameColumnName(value))
                {
                    return;
                }

                this.displayMember = value;

                RebuildColumns();
            }
        }

        #endregion
        #region 데이터 소스 - DataSource

        /// <summary>
        /// 데이터 소스
        /// </summary>
        public object DataSource
        {
            get
            {
                return this.dataGridView.DataSource;
            }
            set
            {
                if(value is IList)
                {
                    this.dataGridView.DataSource = value;

                    RebuildColumns();
                }
                else
                {
                    this.dataGridView.DataSource = null;
                }
            }
        }

        #endregion
        #region 읽기 전용 여부 - ReadOnly

        /// <summary>
        /// 읽기 전용 여부
        /// </summary>
        public bool ReadOnly
        {
            get
            {
                return this.readOnly;
            }
            set
            {
                this.readOnly = value;
            }
        }

        #endregion
        #region 이용 가능 여부 - Enabled

        /// <summary>
        /// 이용 가능 여부
        /// </summary>
        public new bool Enabled
        {
            get
            {
                return base.Enabled;
            }
            set
            {
                this.dataGridView.ClearSelection();

                base.Enabled = value;
            }
        }

        #endregion
        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        public string Value
        {
            get
            {
                if(this.dataGridView.DataSource == null)
                {
                    return null;
                }

                IList list = this.dataGridView.DataSource as IList;

                if(list == null)
                {
                    return null;
                }

                if(this.checkDictionary.Count == 0)
                {
                    return null;
                }

                PropertyDescriptorCollection pdc = TypeDescriptor.GetProperties(list[0].GetType());

                SortedList<string, string> sortedList = new SortedList<string, string>();

                foreach(KeyValuePair<int, object> keyValuePair in this.checkDictionary)
                {
                    string value = pdc[this.valueMember].GetValue(keyValuePair.Value).ToString();

                    if(!sortedList.ContainsKey(value))
                    {
                        sortedList.Add(value, value);
                    }
                }

                StringBuilder stringBuilder = new StringBuilder();

                for(int i = 0; i < sortedList.Count; i++)
                {
                    string value = sortedList[sortedList.Keys[i]];

                    stringBuilder.AppendFormat("{0},", value);
                }

                return stringBuilder.ToString().TrimEnd(',');
            }
            set
            {
                this.checkDictionary.Clear();

                if(this.dataGridView.DataSource == null)
                {
                    return;
                }

                IList list = this.dataGridView.DataSource as IList;

                if(list == null || list.Count == 0)
                {
                    return;
                }

                if(string.IsNullOrWhiteSpace(value))
                {
                    return;
                }

                PropertyDescriptorCollection pdc = TypeDescriptor.GetProperties(list[0].GetType());

                string[] elementValueArray = value.Split(',');
                int      elementValueCount = elementValueArray.Length;

                string elementValue = elementValueArray[0].Trim();

                for(int j = 0; j < list.Count; j++)
                {
                    string itemValue = pdc[this.valueMember].GetValue(list[j]).ToString();

                    if(elementValue == itemValue)
                    {
                        if(!this.checkDictionary.ContainsKey(j))
                        {
                            this.checkDictionary.Add(j, list[j]);
                        }
                    }
                }

                this.dataGridView.Refresh();
            }
        }

        #endregion
        #region 단일 선택 여부 - SingleSelect

        /// <summary>
        /// 단일 선택 여부
        /// </summary>
        public bool SingleSelect
        {
            get
            {
                return this.singleSelect;
            }
            set
            {
                if(this.singleSelect == value)
                {
                    return;
                }

                this.singleSelect = value;

                if(this.singleSelect)
                {
                    string lastValue = Value;

                    if(string.IsNullOrEmpty(lastValue))
                    {
                        return;
                    }

                    string[] elementValueArray = lastValue.Split(',');

                    Value = elementValueArray[0];
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ListBoxControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public ListBoxControl()
        {
            InitializeComponent();

            this.checkDictionary = new Dictionary<int, object>();

            UIHelper.Initialize(this.dataGridView);

            VisibleChanged                   += UserControl_VisibleChanged;
            this.dataGridView.SizeChanged    += dataGridView_SizeChanged;
            this.dataGridView.MouseDown      += dataGridView_MouseDown;
            this.dataGridView.MouseMove      += dataGridView_MouseMove;
            this.dataGridView.MouseUp        += dataGridView_MouseUp;
            this.dataGridView.CellFormatting += dataGridView_CellFormatting;

        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 사용자 컨트롤 표시 여부 변경시 처리하기 - UserControl_VisibleChanged(sender, e)

        /// <summary>
        /// 사용자 컨트롤 표시 여부 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void UserControl_VisibleChanged(object sender, EventArgs e)
        {
            this.dataGridView.ClearSelection();
        }

        #endregion
        #region 데이터 그리드 뷰 크기 변경시 처리하기 - dataGridView_SizeChanged(sender, e)

        /// <summary>
        /// 데이터 그리드 뷰 크기 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void dataGridView_SizeChanged(object sender, EventArgs e)
        {
            if(this.dataGridView.Columns.Contains(_textColumnName))
            {
                this.dataGridView.Columns[_textColumnName].Width = this.dataGridView.ClientSize.Width - 40;
            }
        }

        #endregion
        #region 데이터 그리드 뷰 마우스 DOWN 처리하기 - dataGridView_MouseDown(sender, e)

        /// <summary>
        /// 데이터 그리드 뷰 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void dataGridView_MouseDown(object sender, MouseEventArgs e)
        {
            DataGridView dataGridView = sender as DataGridView;

            DataGridView.HitTestInfo hitTestInfo = dataGridView.HitTest(e.Location.X, e.Location.Y);

            if(hitTestInfo.Type == DataGridViewHitTestType.Cell)
            {
                this.isMouseDragging = true;

                this.dragStartIndex = hitTestInfo.RowIndex;
            }
        }

        #endregion
        #region 데이터 그리드 뷰 마우스 이동시 처리하기 - dataGridView_MouseMove(sender, e)

        /// <summary>
        /// 데이터 그리드 뷰 마우스 이동시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void dataGridView_MouseMove(object sender, MouseEventArgs e)
        {
            DataGridView dataGridView = sender as DataGridView;

            DataGridView.HitTestInfo hitTestInfo = dataGridView.HitTest(e.Location.X, e.Location.Y);

            if(hitTestInfo.Type == DataGridViewHitTestType.Cell)
            {
                if(this.isMouseDragging) // 마우스를 드래그 하고 있는 경우 처리한다.
                {
                    #region 드래그 종료 인덱스를 설정한다.

                    this.dragEndIndex = hitTestInfo.RowIndex;

                    DataGridViewSelectedRowCollection collection = dataGridView.SelectedRows;

                    if(this.dragEndIndex < 0)
                    {
                        if(this.dragStartIndex == collection[0].Index)
                        {
                            this.dragEndIndex = collection[collection.Count - 1].Index;
                        }
                        else
                        {
                            this.dragEndIndex = collection[0].Index;
                        }
                    }

                    #endregion
                    #region 시작/종료 인덱스를 설정한다.

                    int startIndex;
                    int endIndex;

                    if(this.dragStartIndex > this.dragEndIndex)
                    {
                        startIndex = this.dragEndIndex;
                        endIndex   = this.dragStartIndex;
                    }
                    else
                    {
                        startIndex = this.dragStartIndex;
                        endIndex   = this.dragEndIndex;
                    }

                    #endregion
                    
                    dataGridView.ClearSelection();

                    for(int i = startIndex; i <= endIndex; i++)
                    {
                        this.dataGridView.Rows[i].Selected = true;
                    }
                }
            }
        }

        #endregion
        #region 데이터 그리드 뷰 마우스 UP 처리하기 - dataGridView_MouseUp(sender, e)

        /// <summary>
        /// 데이터 그리드 뷰 마우스 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void dataGridView_MouseUp(object sender, MouseEventArgs e)
        {
            DataGridView dataGridView = sender as DataGridView;

            DataGridView.HitTestInfo hitTestInfo = dataGridView.HitTest(e.Location.X, e.Location.Y);

            if(hitTestInfo.Type == DataGridViewHitTestType.Cell)
            {
                if(this.isMouseDragging) // 마우스를 드래그 하고 있는 경우 처리한다.
                {
                    this.isMouseDragging = false;

                    #region 드래그 종료 인덱스를 설정한다.

                    this.dragEndIndex = hitTestInfo.RowIndex;

                    DataGridViewSelectedRowCollection collection = dataGridView.SelectedRows;

                    if(this.dragEndIndex < 0)
                    {
                        if(this.dragStartIndex == collection[0].Index)
                        {
                            this.dragEndIndex = collection[collection.Count - 1].Index;
                        }
                        else
                        {
                            this.dragEndIndex = collection[0].Index;
                        }
                    }

                    #endregion
                    #region 시작/종료 인덱스를 설정한다.

                    int  startIndex;
                    int  endIndex;
                    bool bottomDirection;

                    if(this.dragStartIndex > this.dragEndIndex)
                    {
                        startIndex = this.dragEndIndex;
                        endIndex   = this.dragStartIndex;

                        bottomDirection = false;
                    }
                    else
                    {
                        startIndex = this.dragStartIndex;
                        endIndex   = this.dragEndIndex;

                        bottomDirection = true;
                    }

                    #endregion
                    #region 선택 범위를 체크하거나 체크 취소한다.

                    try
                    {
                        if(!this.readOnly)
                        {
                            if(this.singleSelect)
                            {
                                if(bottomDirection)
                                {
                                    if(this.checkDictionary.ContainsKey(endIndex))
                                    {
                                        this.checkDictionary.Remove(endIndex);
                                    }
                                    else
                                    {
                                        this.checkDictionary.Clear();

                                        this.checkDictionary.Add(endIndex, dataGridView.Rows[endIndex].DataBoundItem);
                                    }
                                }
                                else
                                {
                                    if(this.checkDictionary.ContainsKey(startIndex))
                                    {
                                        this.checkDictionary.Remove(startIndex);
                                    }
                                    else
                                    {
                                        this.checkDictionary.Clear();

                                        this.checkDictionary.Add(startIndex, dataGridView.Rows[startIndex].DataBoundItem);
                                    }
                                }
                            }
                            else
                            {
                                for(int i = startIndex; i <= endIndex; i++)
                                {
                                    if(this.checkDictionary.ContainsKey(i))
                                    {
                                        this.checkDictionary.Remove(i);
                                    }
                                    else
                                    {
                                        this.checkDictionary.Add(i, dataGridView.Rows[i].DataBoundItem);
                                    }
                                }
                            }
                        }
                    }
                    finally
                    {
                        dataGridView.ClearSelection();

                        dataGridView.Refresh();
                    }

                    #endregion
                }
            }
        }

        #endregion
        #region 데이터 그리드 뷰 셀 포맷 처리하기 - dataGridView_CellFormatting(sender, e)

        /// <summary>
        /// 데이터 그리드 뷰 셀 포맷 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void dataGridView_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            if(e.ColumnIndex == 0)
            {
                if(this.checkDictionary.ContainsKey(e.RowIndex))
                {
                    e.Value = true; 
                }
                else
                {
                    e.Value = false;
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 동일 컬럼명 여부 조사하기 - CheckSameColumnName(columnName)

        /// <summary>
        /// 동일 컬럼명 여부 조사하기
        /// </summary>
        /// <param name="columnName">컬럼명</param>
        /// <returns>동일 컬럼명 여부</returns>
        private bool CheckSameColumnName(string columnName)
        {
            if(string.IsNullOrWhiteSpace(columnName))
            {
                return false;
            }

            if(this.dataGridView.Columns.Count == 0)
            {
                return false;
            }

            return this.dataGridView.Columns.Contains(columnName);
        }

        #endregion
        #region 컬럼 재구축하기 - RebuildColumns()

        /// <summary>
        /// 컬럼 재구축하기
        /// </summary>
        private void RebuildColumns()
        {
            this.dataGridView.Columns.Clear();

            UIHelper.AddCheckBoxColumn(this.dataGridView, "DATAGRIDVIEWCHECKCOLUMN", "", "DATAGRIDVIEWCHECKCOLUMN", 20, true, false, false, true);

            if(!string.IsNullOrWhiteSpace(this.valueMember))
            {
                UIHelper.AddTextColumn(this.dataGridView, _valueColumnName, "", this.valueMember, 100, true, false, false, false);
            }

            if(!string.IsNullOrWhiteSpace(this.displayMember))
            {
                UIHelper.AddTextColumn(this.dataGridView, _textColumnName, "", this.displayMember, 100, true, false, false, true);

                this.dataGridView.Columns[_textColumnName].Width = this.dataGridView.ClientSize.Width - 40;
            }
        }

        #endregion
    }
}