﻿using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// UI 헬퍼
    /// </summary>
    public static class UIHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화 하기 - Initialize(dataGridView)

        /// <summary>
        /// 초기화 하기
        /// </summary>
        /// <param name="dataGridView">데이터 그리드 뷰</param>
        public static void Initialize(DataGridView dataGridView)
        {
            dataGridView.BackgroundColor             = Color.White;

            dataGridView.AutoGenerateColumns         = false;
            dataGridView.ColumnHeadersVisible        = false;
            dataGridView.AutoSizeColumnsMode         = DataGridViewAutoSizeColumnsMode.None;
            dataGridView.ColumnHeadersHeightSizeMode = DataGridViewColumnHeadersHeightSizeMode.DisableResizing;

            dataGridView.RowHeadersVisible           = false;
            dataGridView.AutoSizeRowsMode            = DataGridViewAutoSizeRowsMode.None;
            dataGridView.RowTemplate.Height          = 18;

            dataGridView.AllowUserToAddRows          = false;
            dataGridView.AllowUserToDeleteRows       = false;
            dataGridView.AllowUserToOrderColumns     = false;
            dataGridView.AllowUserToResizeColumns    = false;
            dataGridView.AllowUserToResizeRows       = false;

            dataGridView.MultiSelect                 = true;
            dataGridView.SelectionMode               = DataGridViewSelectionMode.FullRowSelect;

            dataGridView.GridColor                   = Color.White;
        }

        #endregion

        #region 체크 박스 컬럼 추가하기 - AddCheckBoxColumn(dataGridView, columnName, captionName, fieldName, columnWidth, readOnly, resizable, sortable, visible)

        /// <summary>
        /// 체크 박스 컬럼 추가하기
        /// </summary>
        /// <param name="dataGridView">데이터 그리드 뷰</param>
        /// <param name="columnName">컬럼명</param>
        /// <param name="captionName">제목명</param>
        /// <param name="fieldName">필드명</param>
        /// <param name="columnWidth">컬럼 너비</param>
        /// <param name="readOnly">읽기 전용 여부</param>
        /// <param name="resizable">크기 변경 가능 여부</param>
        /// <param name="sortable">정렬 가능 여부</param>
        /// <param name="visible">표시 여부</param>
        /// <returns>컬럼 인덱스</returns>
        public static int AddCheckBoxColumn(DataGridView dataGridView, string columnName, string captionName, string fieldName, int columnWidth, bool readOnly, bool resizable, bool sortable, bool visible)
        {
            DataGridViewCheckBoxColumn column = new DataGridViewCheckBoxColumn();

            column.Name             = columnName;
            column.HeaderText       = captionName;
            column.DataPropertyName = fieldName;
            column.Width            = columnWidth;
            column.ReadOnly         = readOnly;
            column.Resizable        = resizable ? DataGridViewTriState.True : DataGridViewTriState.False;
            column.SortMode         = sortable ? DataGridViewColumnSortMode.Automatic : DataGridViewColumnSortMode.NotSortable;
            column.Visible          = visible;

            int index = dataGridView.Columns.Add(column);

            return index;
        }

        #endregion
        #region 이미지 컬럼 추가하기 - AddImageColumn(dataGridView, columnName, captionName, fieldName, columnWidth, readOnly, resizable, sortable, visible)

        /// <summary>
        /// 이미지 컬럼 추가하기
        /// </summary>
        /// <param name="dataGridView">데이터 그리드 뷰</param>
        /// <param name="columnName">컬럼명</param>
        /// <param name="captionName">제목명</param>
        /// <param name="fieldName">필드명</param>
        /// <param name="columnWidth">컬럼 너비</param>
        /// <param name="readOnly">읽기 전용 여부</param>
        /// <param name="resizable">크기 변경 가능 여부</param>
        /// <param name="sortable">정렬 가능 여부</param>
        /// <param name="visible">표시 여부</param>
        /// <returns>컬럼 인덱스</returns>
        public static int AddImageColumn(DataGridView dataGridView, string columnName, string captionName, string fieldName, int columnWidth, bool readOnly, bool resizable, bool sortable, bool visible)
        {
            DataGridViewImageColumn column = new DataGridViewImageColumn();

            column.Name             = columnName;
            column.HeaderText       = captionName;
            column.DataPropertyName = fieldName;
            column.Width            = columnWidth;
            column.ReadOnly         = readOnly;
            column.Resizable        = resizable ? DataGridViewTriState.True : DataGridViewTriState.False;
            column.SortMode         = sortable ? DataGridViewColumnSortMode.Automatic : DataGridViewColumnSortMode.NotSortable;
            column.Visible          = visible;

            int index = dataGridView.Columns.Add(column);

            return index;
        }

        #endregion
        #region 텍스트 컬럼 추가하기 - AddTextColumn(dataGridView, columnName, captionName, fieldName, columnWidth, readOnly, resizable, sortable, visible)

        /// <summary>
        /// 텍스트 컬럼 추가하기
        /// </summary>
        /// <param name="dataGridView">데이터 그리드 뷰</param>
        /// <param name="columnName">컬럼명</param>
        /// <param name="captionName">제목명</param>
        /// <param name="fieldName">필드명</param>
        /// <param name="columnWidth">컬럼 너비</param>
        /// <param name="readOnly">읽기 전용 여부</param>
        /// <param name="resizable">크기 변경 가능 여부</param>
        /// <param name="sortable">정렬 가능 여부</param>
        /// <param name="visible">표시 여부</param>
        /// <returns>컬럼 인덱스</returns>
        public static int AddTextColumn(DataGridView dataGridView, string columnName, string captionName, string fieldName, int columnWidth, bool readOnly, bool resizable, bool sortable, bool visible)
        {
            DataGridViewTextBoxColumn column = new DataGridViewTextBoxColumn();

            column.Name             = columnName;
            column.HeaderText       = captionName;
            column.DataPropertyName = fieldName;
            column.Width            = columnWidth;
            column.ReadOnly         = readOnly;
            column.Resizable        = resizable ? DataGridViewTriState.True : DataGridViewTriState.False;
            column.SortMode         = sortable ? DataGridViewColumnSortMode.Automatic : DataGridViewColumnSortMode.NotSortable;
            column.Visible          = visible;

            int index = dataGridView.Columns.Add(column);

            return index;
        }

        #endregion
    }
}